/* Copyright(c) 2003++ Agilis Software LLC.  All rights reserved. */

/* IlsPlat.h: Ils platform independence header file */

/*
 * This header file is automatically included when you include
 * IlsClient.h.
 * If you are using the supplied client library, the contents of this
 * file are irrelevant.  Otherwise,
 * From the perspective of the application developer, the
 * following are relevant:
 * CALLING_CONVENTION macro: controls calling convention on Windows:
 *		sets it to __stdcall for the benefit of non-C application
 *	    environments (and of course it works for C as well).
 *   How to use: prefix your API calls with this if you're defining your
 *		own.  Change to "cdecl" for Windows if you want the traditional
 *	    C calling convention.  Or simply leave alone.
 *		Not relevant for normal platforms.
 * DLLEXPORT macro: controls, on Windows, whether an API call is
 *		visible in your DLL.
 *   How to use: prefix your public API calls with this macro if you're
 * 		defining your own.
 *		Not relevant for normal platforms.
 * "boolean" macro: platform/compiler-safe way of defining a boolean
 *	    data type.
 *   How to use: use this instead of platform / compiler-specific
 *	    "bool" etc. if available, to be safe for portability.
 * "TRUE", "FALSE" macros: values for "boolean" datatype above.
 * "NULL": to define null value if it isn't available for your platform / compiler.
 * PATH_SEPARATOR: path separator character in PATH / LD_LIBRARY_PATH environment variable
 * FILE_SEPARATOR: file path separator character in fully qualified file name specification.
 * LIBPATH_ENV_NAME: name of environment variable that encompasses shared libraries
 * SL_FILE_EXT: file extension for shared-library file
 * SOCKET: defined to int if not Windows
 */

#ifndef __ILSPLAT_H_
#define __ILSPLAT_H_

#if !defined(_WINDOWS)
#if defined(WIN32) || defined(WIN64)
#	define _WINDOWS
#	define _CRT_SECURE_NO_DEPRECATE 1
#	define _SCL_SECURE_NO_DEPRECATE 1
#endif
#endif

#if !defined(_LINUX)
#if defined(LINUX)
#	define _LINUX
#endif
#elif !defined(LINUX)
#	define LINUX
#endif

#if defined (_VXWORKS)
#       include <vxWorks.h>
#       include <hostLib.h>
#       include <ioLib.h>
#       include <sockLib.h>
#       include <taskLib.h>
#       include <sysLib.h>
#endif

#if !defined(_WINDOWS)
#	include <unistd.h>
#	include <sys/stat.h>
#	include <sys/socket.h>
#  if !defined(_HPUX) &&  !defined (_VXWORKS)
#	include <sys/select.h>
#  else
#       if defined (_VXWORKS)
#	   include <time.h>
#       else
#	   include <sys/time.h>
#       endif
#  endif
#	include <errno.h>
/* if defined(LINUX) */
#  if !defined (_VXWORKS)
#	include <sys/utsname.h>
#  endif
#	include <netinet/in.h>
/* endif */
#else
#	include <windows.h>
#	include <winsock.h>
#	include <winbase.h>
#	include <io.h>
#	include <sys/stat.h>
#endif

/* Bare-bones POSIX thread abstraction (just what's needed for now (basic mutexing for thread safety)) */
#if defined(_WINDOWS)
#	define PTHREAD_MUTEX_T			CRITICAL_SECTION
#	define PTHREAD_MUTEX_INIT(m)	InitializeCriticalSection(m),0
#	define PTHREAD_MUTEX_DESTROY(m)	DeleteCriticalSection(m),0
#	define PTHREAD_MUTEX_LOCK(m)	EnterCriticalSection(m),0
#	define PTHREAD_MUTEX_UNLOCK(m)	LeaveCriticalSection(m),0
#elif defined(_SUN) || defined(_HPUX) || defined(_CYGWIN) || defined(_LINUX) || defined(MACOSX)
#	include <pthread.h>
#	define PTHREAD_MUTEX_T			pthread_mutex_t
#	define PTHREAD_MUTEX_INIT(m)	pthread_mutex_init(m, NULL)
#	define PTHREAD_MUTEX_DESTROY(m)	pthread_mutex_destroy(m)
#	define PTHREAD_MUTEX_LOCK(m)	pthread_mutex_lock(m)
#	define PTHREAD_MUTEX_UNLOCK(m)	pthread_mutex_unlock(m)
#else
/* No-op everything - no threads, no thread-safety, just ensure everything compiles */
#	define PTHREAD_MUTEX_T			int
#	define PTHREAD_MUTEX_INIT(m) {;}
#	define PTHREAD_MUTEX_DESTROY(m) {;}
#	define PTHREAD_MUTEX_LOCK(m) {;}
#	define PTHREAD_MUTEX_UNLOCK(m) {;}
#endif

#if defined(_WINDOWS)
#	define CALLING_CONVENTION __stdcall
#	define DLLEXPORT __declspec( dllexport )
#elif defined(_CYGWIN)
#	define CALLING_CONVENTION __stdcall
#	define DLLEXPORT
#else
#	define CALLING_CONVENTION
#	define DLLEXPORT
#endif

#if !defined(_WINDOWS)
#   if defined (_VXWORKS)
#		define Sleep(ms)	taskDelay(ms * (sysClkRateGet()/1000))
#		define sleepSec(sec)	taskDelay(sec*sysClkRateGet())
#		define SOCKET	int
#       define gethostbyname(name)    hostGetByName(name)
#		define NO_CLOCK
#		define NO_FILESYSTEM
#		define MINIMIZE_FOOTPRINT
#   else
#		define Sleep(ms)	sleep((ms)/1000)
#		define sleepSec(sec)	sleep(sec)
#		define SOCKET	int
#   endif
#else
#	define sleepSec(sec)	Sleep((sec)*1000)
#endif

typedef unsigned char boolean;
#define TRUE 1
#define FALSE 0

#if !defined(NULL)
#	define NULL 0
#endif

#if defined(_WINDOWS)
#	define PATH_SEPARATOR	";"
#else
#	define PATH_SEPARATOR	":"
#endif

#if defined(_SUN)
#	define LIBPATH_ENV_NAME "LD_LIBRARY_PATH"
#else
#	define LIBPATH_ENV_NAME "PATH"
#endif

#if defined(_WINDOWS)
#	define SL_FILE_EXT ".dll"
#elif defined(_HPUX)
#	define SL_FILE_EXT ".sl"
#else
#	define SL_FILE_EXT ".so"
#endif

#if defined(_WINDOWS)
#	define FILE_SEPARATOR "\\"
#else
#	define FILE_SEPARATOR "/"
#endif

#endif
